@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -36,27 +34,25 @@ infeasible.  We may also be quite aggressive in bumping the minimum version of
 Lark, unless someone volunteers to either fund or do the maintainence.
 """
 
-from __future__ import absolute_import, division, print_function
+from inspect import getfullargspec
+from typing import Dict
 
 import attr
 import lark
 from lark.grammar import NonTerminal, Terminal
 
-import hypothesis._strategies as st
+import hypothesis.strategies._internal.core as st
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import getfullargspec, string_types
 from hypothesis.internal.conjecture.utils import calc_label_from_name
+from hypothesis.internal.reflection import deprecated_posargs
 from hypothesis.internal.validation import check_type
-from hypothesis.searchstrategy import SearchStrategy
-
-if False:
-    from typing import Dict, Text  # noqa
+from hypothesis.strategies._internal import SearchStrategy
 
 __all__ = ["from_lark"]
 
 
 @attr.s()
-class DrawState(object):
+class DrawState:
     """Tracks state of a single draw from a lark grammar.
 
     Currently just wraps a list of tokens that will be emitted at the
@@ -149,7 +145,7 @@ class LarkStrategy(SearchStrategy):
         state = DrawState()
         start = data.draw(self.start)
         self.draw_symbol(data, start, state)
-        return u"".join(state.result)
+        return "".join(state.result)
 
     def rule_label(self, name):
         try:
@@ -191,7 +187,7 @@ class LarkStrategy(SearchStrategy):
 
 def check_explicit(name):
     def inner(value):
-        check_type(string_types, value, "value drawn from " + name)
+        check_type(str, value, "value drawn from " + name)
         return value
 
     return inner
@@ -199,12 +195,13 @@ def check_explicit(name):
 
 @st.cacheable
 @st.defines_strategy_with_reusable_values
+@deprecated_posargs
 def from_lark(
-    grammar,  # type: lark.lark.Lark
-    start=None,  # type: Text
-    explicit=None,  # type: Dict[Text, st.SearchStrategy[Text]]
-):
-    # type: (...) -> st.SearchStrategy[Text]
+    grammar: lark.lark.Lark,
+    *,
+    start: str = None,
+    explicit: Dict[str, st.SearchStrategy[str]] = None
+) -> st.SearchStrategy[str]:
     """A strategy for strings accepted by the given context-free grammar.
 
     ``grammar`` must be a ``Lark`` object, which wraps an EBNF specification.
