@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -19,15 +17,13 @@
 obviously belong anywhere else. If you spot a better home for
 anything that lives here, please move it."""
 
+import array
 
-from __future__ import absolute_import, division, print_function
 
-from hypothesis.internal.compat import (
-    array_or_list,
-    hbytes,
-    int_to_bytes,
-    integer_types,
-)
+def array_or_list(code, contents):
+    if code == "O":
+        return list(contents)
+    return array.array(code, contents)
 
 
 def replace_all(buffer, replacements):
@@ -46,14 +42,14 @@ def replace_all(buffer, replacements):
         offset += len(r) - (v - u)
     result.extend(buffer[prev:])
     assert len(result) == len(buffer) + offset
-    return hbytes(result)
+    return bytes(result)
 
 
 ARRAY_CODES = ["B", "H", "I", "L", "Q", "O"]
 NEXT_ARRAY_CODE = dict(zip(ARRAY_CODES, ARRAY_CODES[1:]))
 
 
-class IntList(object):
+class IntList:
     """Class for storing a list of non-negative integers compactly.
 
     We store them as the smallest size integer array we can get
@@ -71,12 +67,19 @@ class IntList(object):
             except OverflowError:
                 pass
         else:  # pragma: no cover
-            assert False, "Could not create storage for %r" % (values,)
+            raise AssertionError("Could not create storage for %r" % (values,))
         if isinstance(self.__underlying, list):
             for v in self.__underlying:
-                if v < 0 or not isinstance(v, integer_types):
+                if v < 0 or not isinstance(v, int):
                     raise ValueError("Could not create IntList for %r" % (values,))
 
+    @classmethod
+    def of_length(self, n):
+        return IntList(array_or_list("B", [0]) * n)
+
+    def count(self, n):
+        return self.__underlying.count(n)
+
     def __repr__(self):
         return "IntList(%r)" % (list(self),)
 
@@ -109,33 +112,28 @@ class IntList(object):
         return self.__underlying != other.__underlying
 
     def append(self, n):
+        i = len(self)
+        self.__underlying.append(0)
+        self[i] = n
+
+    def __setitem__(self, i, n):
         while True:
             try:
-                self.__underlying.append(n)
+                self.__underlying[i] = n
                 return
             except OverflowError:
                 assert n > 0
                 self.__upgrade()
 
+    def extend(self, ls):
+        for n in ls:
+            self.append(n)
+
     def __upgrade(self):
         code = NEXT_ARRAY_CODE[self.__underlying.typecode]
         self.__underlying = array_or_list(code, self.__underlying)
 
 
-def pop_random(random, values):
-    """Remove a random element of values, possibly changing the ordering of its
-    elements."""
-
-    # We pick the element at a random index. Rather than removing that element
-    # from the list (which would be an O(n) operation), we swap it to the end
-    # and return the last element of the list. This changes the order of
-    # the elements, but as long as these elements are only accessed through
-    # random sampling that doesn't matter.
-    i = random.randrange(0, len(values))
-    values[i], values[-1] = values[-1], values[i]
-    return values.pop()
-
-
 def binary_search(lo, hi, f):
     """Binary searches in [lo , hi) to find
     n such that f(n) == f(lo) but f(n + 1) != f(lo).
@@ -155,5 +153,66 @@ def binary_search(lo, hi, f):
 
 
 def uniform(random, n):
-    """Returns an hbytes of length n, distributed uniformly at random."""
-    return int_to_bytes(random.getrandbits(n * 8), n)
+    """Returns a bytestring of length n, distributed uniformly at random."""
+    return random.getrandbits(n * 8).to_bytes(n, "big")
+
+
+class LazySequenceCopy:
+    """A "copy" of a sequence that works by inserting a mask in front
+    of the underlying sequence, so that you can mutate it without changing
+    the underlying sequence. Effectively behaves as if you could do list(x)
+    in O(1) time. The full list API is not supported yet but there's no reason
+    in principle it couldn't be."""
+
+    def __init__(self, values):
+        self.__values = values
+        self.__len = len(values)
+        self.__mask = None
+
+    def __len__(self):
+        return self.__len
+
+    def pop(self):
+        if len(self) == 0:
+            raise IndexError("Cannot pop from empty list")
+        result = self[-1]
+        self.__len -= 1
+        if self.__mask is not None:
+            self.__mask.pop(self.__len, None)
+        return result
+
+    def __getitem__(self, i):
+        i = self.__check_index(i)
+        default = self.__values[i]
+        if self.__mask is None:
+            return default
+        else:
+            return self.__mask.get(i, default)
+
+    def __setitem__(self, i, v):
+        i = self.__check_index(i)
+        if self.__mask is None:
+            self.__mask = {}
+        self.__mask[i] = v
+
+    def __check_index(self, i):
+        n = len(self)
+        if i < -n or i >= n:
+            raise IndexError("Index %d out of range [0, %d)" % (i, n))
+        if i < 0:
+            i += n
+        assert 0 <= i < n
+        return i
+
+
+def clamp(lower, value, upper):
+    """Given a value and lower/upper bounds, 'clamp' the value so that
+    it satisfies lower <= value <= upper."""
+    return max(lower, min(value, upper))
+
+
+def swap(ls, i, j):
+    """Swap the elements ls[i], ls[j]."""
+    if i == j:
+        return
+    ls[i], ls[j] = ls[j], ls[i]
